/**
 * Interface which describes URI components.
 */
export interface IUri {
    Protocol: string;
    HostNameAndPort: string;
    AbsolutePath: string;
    Search: string;
    Hash: string;
    PathSegments: string[];
    QueryString: string;
}
//# sourceMappingURL=IUri.d.ts.map