/**
 * Controls whether tokens should be stored in the cache or not. If set to false, tokens may still be acquired and returned but will not be cached for later retrieval.
 */
export type StoreInCache = {
    accessToken?: boolean;
    idToken?: boolean;
    refreshToken?: boolean;
};
//# sourceMappingURL=StoreInCache.d.ts.map