/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

/**
 * Key-Value type to support queryParams, extraQueryParams and claims
 */
export type StringDict = { [key: string]: string };
