/**
 * Authority types supported by MSAL.
 */
export declare const AuthorityType: {
    readonly Default: 0;
    readonly Adfs: 1;
    readonly Dsts: 2;
    readonly Ciam: 3;
};
export type AuthorityType = (typeof AuthorityType)[keyof typeof AuthorityType];
//# sourceMappingURL=AuthorityType.d.ts.map