/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

export type CloudDiscoveryMetadata = {
    preferred_network: string;
    preferred_cache: string;
    aliases: Array<string>;
};
