export type ServerTelemetryEntity = {
    failedRequests: Array<string | number>;
    errors: string[];
    cacheHits: number;
    nativeBrokerErrorCode?: string;
};
//# sourceMappingURL=ServerTelemetryEntity.d.ts.map