/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

export type NetworkResponse<T> = {
    headers: Record<string, string>;
    body: T;
    status: number;
};
