/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

/**
 * App Metadata Cache Type
 */
export type AppMetadataEntity = {
    /** clientId of the application */
    clientId: string;
    /** entity that issued the token, represented as a full host */
    environment: string;
    /** Family identifier, '1' represents Microsoft Family */
    familyId?: string;
};
