import { CredentialEntity } from "./CredentialEntity.js";
/**
 * Refresh Token Cache Type
 */
export type RefreshTokenEntity = CredentialEntity & {
    expiresOn?: string;
};
//# sourceMappingURL=RefreshTokenEntity.d.ts.map