/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

export type ServerTelemetryEntity = {
    failedRequests: Array<string | number>;
    errors: string[];
    cacheHits: number;
    nativeBrokerErrorCode?: string;
};
