/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

export interface ISerializableTokenCache {
    deserialize: (cache: string) => void;
    serialize: () => string;
}
