/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

export interface IGuidGenerator {
    generateGuid(): string;
    isGuid(guid: string): boolean;
}
