/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

import { CredentialEntity } from "./CredentialEntity.js";

/**
 * Refresh Token Cache Type
 */
export type RefreshTokenEntity = CredentialEntity & {
    expiresOn?: string;
};
