/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

/**
 * Interface which describes URI components.
 */
export interface IUri {
    Protocol: string;
    HostNameAndPort: string;
    AbsolutePath: string;
    Search: string;
    Hash: string;
    PathSegments: string[];
    QueryString: string;
}
