export type CcsCredential = {
    credential: string;
    type: CcsCredentialType;
};
export declare const CcsCredentialType: {
    readonly HOME_ACCOUNT_ID: "home_account_id";
    readonly UPN: "UPN";
};
export type CcsCredentialType = (typeof CcsCredentialType)[keyof typeof CcsCredentialType];
//# sourceMappingURL=CcsCredential.d.ts.map