/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

import {
    RegionDiscoveryOutcomes,
    RegionDiscoverySources,
} from "../utils/Constants.js";

export type RegionDiscoveryMetadata = {
    region_used?: string;
    region_source?: RegionDiscoverySources;
    region_outcome?: RegionDiscoveryOutcomes;
};
