/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

export type ServerTelemetryRequest = {
    clientId: string;
    apiId: number;
    correlationId: string;
    forceRefresh?: boolean;
    wrapperSKU?: string;
    wrapperVer?: string;
};
