/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

// The type here is a string instead of enum as the list of regional end points supported is not final yet.
export type AzureRegion = string;
