import { ServerResponseType } from "../utils/Constants.js";
/**
 * Options for the OIDC protocol mode.
 */
export type OIDCOptions = {
    serverResponseType?: ServerResponseType;
    defaultScopes?: Array<string>;
};
//# sourceMappingURL=OIDCOptions.d.ts.map